"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const pg_1 = require("pg");
const to_safe_table_name_1 = require("./to-safe-table-name");
const setup_table_1 = require("./setup-table");
const listen_to_table_1 = require("./listen-to-table");
exports.createDb = (config) => __awaiter(this, void 0, void 0, function* () {
    const workerId = Math.round(Math.random() * 30000);
    const Id = config.Id;
    const tablePrefix = config.tablePrefix || 'actions_';
    const snapshotFrequency = config.snapshotFrequency || 100;
    const table = to_safe_table_name_1.toSafeTableName(`${tablePrefix}${Id}`);
    const pool = new pg_1.Pool(config);
    let lastActionId = 0;
    let intervalId;
    let state;
    const conn = yield pool.connect();
    yield setup_table_1.setupTable(pool, table);
    yield listen_to_table_1.listenToTable(pool, table);
    const db = {
        dispatch: (action) => {
            const strAction = JSON.stringify(action);
            return pool.query(`
        INSERT INTO ${table} (action, worker_id)
        VALUES ($1::jsonb, $2::smallint)
      `, [strAction, workerId]);
        },
        close: () => {
            clearInterval(intervalId);
        }
    };
    const emitRow = (row) => {
        const action = JSON.parse(row.action);
        action.$timeMs = row.sent_at;
        lastActionId = row.action_id;
        if (db.onaction) {
            state = db.onaction(action);
        }
    };
    const snapshot = () => __awaiter(this, void 0, void 0, function* () {
        const snapshotAction = { type: 'STATE', state };
        return pool.query(`
      BEGIN;
      DELETE FROM ${table}
      WHERE action_id < $2::integer;
      UPDATE ${table}
      SET action = $1::jsonb
      WHERE action_id = $2::integer;
      COMMIT;
    `, [snapshotAction, lastActionId]);
    });
    const onNotification = (msg) => __awaiter(this, void 0, void 0, function* () {
        if (msg.channel === `${table}_insert`) {
            const row = JSON.parse(msg.payload);
            emitRow(row);
            if (row % snapshotFrequency === 0 && row.worker_id === workerId) {
                try {
                    yield snapshot();
                }
                catch (e) {
                    console.log(e.message);
                }
            }
        }
    });
    conn.on('notification', onNotification);
    const loadHistory = () => __awaiter(this, void 0, void 0, function* () {
        const result = yield pool.query(`
      SELECT *
      FROM ${table}
      ORDER BY action_id ASC
    `, [lastActionId]);
        return result.rows;
    });
    const rows = yield loadHistory();
    rows.forEach(emitRow);
    return db;
});
