"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupTable = (pool, table) => __awaiter(this, void 0, void 0, function* () {
    const client = yield pool.connect();
    try {
        // begin transaction
        yield client.query('BEGIN');
        // create table if it doesn't exist
        yield client.query(`
      CREATE TABLE IF NOT EXISTS ${table} (
        action_id serial    NOT NULL,
        worker_id smallint  NOT NULL,
        sent_at   timestamp NOT NULL DEFAULT NOW(),
        action    jsonb     NOT NULL
      )`);
        // notify us whenever a row is inserted
        yield client.query(`
      CREATE OR REPLACE FUNCTION ${table}_insert_notify()
      RETURNS trigger AS $$
        BEGIN
        PERFORM pg_notify('${table}_insert', row_to_json(NEW)::text);
        RETURN NEW;
        END;
      $$ LANGUAGE plpgsql`);
        // drop the old trigger
        // there is no CREATE OR REPLACE TRIGGER for some reason
        yield client.query(`
      DROP TRIGGER ${table}_insert_trigger ON users
    `);
        // trigger
        yield client.query(`
      CREATE TRIGGER ${table}_insert_trigger AFTER INSERT ON users
      FOR EACH ROW EXECUTE PROCEDURE ${table}_insert_notify()
    `);
        // done
        yield client.query('COMMIT');
    }
    catch (e) {
        yield client.query('ROLLBACK');
        throw e;
    }
    finally {
        client.release();
    }
});
